;;; package --- Imperative and Object-Oriented Programming Methodology
;; Uppsala University - 2018 Autumn, Copyright © 2018 Henrik Alfken(Schulze)
;; This is NOT the official Emacs init file for the course IOOPM.
;;; Commentary:
;; ! Do you get "Error (use-package)"? Try <META-x> package-refresh-contents! Then restart eamcs.
;; Inspired by: http://jamiecollinson.com/blog/my-emacs-config/
;; http://homepages.inf.ed.ac.uk/s0243221/emacs/
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;;; Code:
(setq w32-pipe-read-delay 0)
; ^^ http://www.reddit.com/r/emacs/comments/2iyoj6/emacs_windows_version_slow_when_transferring/
;; http://emacs.stackexchange.com/questions/1/are-there-any-advantages-to-using-emacs-d-init-el-instead-of-emacs#6 ;
(setq package--init-file-ensured t)
;; http://www.reddit.com/r/emacs/comments/56fvgd/is_there_a_way_to_stop_emacs_from_adding_the/
(setq whitespace-line-column 116) (global-linum-mode t) (setq column-number-mode t) ;(global-whitespace-mode 1)
;; ^^ http://stackoverflow.com/questions/6378831/emacs-globally-enable-whitespace-mode#6379539
;; ^^ http://www.emacswiki.org/emacs/HighlightLongLines
(setq redisplay-dont-pause t  scroll-margin 1  scroll-step 1
  scroll-conservatively 99  scroll-preserve-screen-position 1)
;^^ http://stackoverflow.com/questions/3631220/fix-to-get-smooth-scrolling-in-emacs#4160949
(setq inhibit-startup-message t) ;(setq initial-scratch-message "")
(fset 'yes-or-no-p 'y-or-n-p) (scroll-bar-mode -1) (tool-bar-mode -1) ;(menu-bar-mode -1)
;^^ http://emacs.stackexchange.com/questions/23773/disable-scrollbar-on-new-frame/23774#23774
(setq display-time-day-and-date t display-time-24hr-format t) (display-time) (show-paren-mode 1)
;; ^^ http://www.emacswiki.org/emacs/CancelingInEmacs
;; ^^ http://www.gnu.org/software/emacs/manual/html_node/efaq/Matching-parentheses.html
(setq-default major-mode 'text-mode) (setq sentence-end-double-space nil)
;; ^^ http://stackoverflow.com/questions/18172728/the-difference-between-setq-and-setq-default-in-emacs-lisp#18173666
(prefer-coding-system 'utf-8) (set-language-environment "UTF-8") (setq delete-selection-mode 1)
;; http://ergoemacs.org/emacs_manual/emacs/Major-Modes.html#index-major_002dmode
;; http://emacs.stackexchange.com/questions/32205/delete-selection-mode-not-a-default#32208
(setq-default tab-width 2) (setq sh-basic-offset 2) (setq-default indent-tabs-mode nil)
(setq-default set-mark-command-repeat-pop t) (setq imenu-auto-rescan t)
(global-auto-revert-mode 1)
;^^ https://www.emacswiki.org/emacs/AutoRevertMode
(make-directory "~/.emacs.d/files-autosaved/" t)
;^ http://www.reddit.com/r/emacs/comments/8jrp5a/how_to_check_if_a_list_of_directories_exist_and
(setq auto-save-file-name-transforms '((".*" "~/.emacs.d/files-autosaved/" t))
 backup-by-copying t  backup-directory-alist '(("." . "~/.emacs.d/file-backups/"))
 delete-old-versions t  kept-new-versions 20  kept-old-versions 10  version-control t
 large-file-warning-threshold 1000000)
; http://emacs.stackexchange.com/questions/17210/how-to-place-all-auto-save-files-in-a-directory
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;;; init-use-package.el --- Get started with use-package in emacs!
;; http://cachestocaches.com/2015/8/getting-started-use-package/
;http://github.com/CachesToCaches/getting_started_with_use_package/blob/master/init-use-package.el
;; Copyright (C) 2015 Gregory J Stein
;; Author: Gregory J Stein <gregory.j.stein@gmail.com>; Henrik Alfken <henke.uu.se@inbox.lv>
;; Code inspired by: http://stackoverflow.com/a/10093312/3672986
;;   http://swsnr.de/blog/2015/01/06/my-emacs-configuration-with-use-package/
;;   http://github.com/jwiegley/use-package
;;; Commentary:
;; As Sebastian Wiesner from http://www.lunaryorn.com/ points out, there is a "chicken and egg"
;; problem with use-package, which is capable of automatically downloading and installing
;; packages, but otherwise needs to be downloaded and installed manually. I include the
;; following code in my Emacs initialization file so that this process is automatic.
;;; Code:
;; Update package-archive lists:
(require 'package) (setq package-enable-at-startup nil)
(add-to-list 'package-archives '("melpa" . "http://melpa.org/packages/"))
(add-to-list 'package-archives '("marmalade" . "https://marmalade-repo.org/packages/"))
(add-to-list 'package-archives '("gnu" . "http://elpa.gnu.org/packages/"))
(package-initialize)
;(package-refresh-contents) ; !Uncomment this line if use-package gives an Error: "Not Found"
;; http://stackoverflow.com/questions/24833964/package-listed-in-melpa-but-not-found-in-package-install#29017473
(unless (package-installed-p 'use-package) (package-refresh-contents)
 (package-install 'use-package)) ; <<^^ Install 'use-package' if necessary.
(eval-when-compile (require 'use-package)) ;(require 'diminish) ; << Maybe uncomment.
(require 'bind-key) ; << If you use any :bind variant.
;;; init-use-package.el ends here
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
(setq default-frame-alist '((left . -0) (width . 83) (fullscreen . fullheight)
 (font . "DejaVu Sans Mono-8") (vertical-scroll-bars . nil)))
(add-to-list 'default-frame-alist '(background-color . "#0f0f0f"))
;http://emacs.stackexchange.com/questions/31058/how-to-make-size-of-emacs-frame-persistent#31069
;; http://www.reddit.com/r/emacs/comments/9c0a4d/tip_setting_initial_frame_size_and_position/#t1_e573owh
;; ^^ http://www.gnu.org/software/emacs/manual/html_node/emacs/Fonts.html#index-fontconfig-1667
;http://stackoverflow.com/questions/11418648/how-to-set-default-emacs-background-and-foreground-colors#11419691
;http://emacs.stackexchange.com/questions/7481/how-to-evaluate-the-variables-before-adding-them-to-a-list#7487
(use-package monokai-theme :ensure t :defer t :init (load-theme 'monokai t))
 (setq initial-frame-alist '((background-color . "#0f0f0f")))
; IF theme in "~/share/emacs/2x.x/etc/themes" - replace ":ensure t" by ":ensure nil"! /Alfken
;^^http://emacs.stackexchange.com/questions/10396/difference-between-init-and-config-in-use-package#10403
(use-package ace-window :ensure t)
(use-package auto-complete :ensure t) (ac-config-default)
(require 'auto-complete) (require 'auto-complete-config);;
;http://emacs.stackexchange.com/questions/14505/how-to-make-cedet-auto-completion-more-detailed
(use-package avy :ensure t)
;(use-package counsel :ensure t :bind ("M-x" . counsel-M-x))
;^^DON'T USE counsel! - "Wrong number of arguments: setq, 1"
(use-package crux :ensure t)
(use-package expand-region :ensure t)
(use-package ggtags :ensure t)
(add-hook 'c-mode-common-hook (lambda () (when (derived-mode-p 'c-mode 'c++-mode 'java-mode)
 (ggtags-mode 1))))
(use-package flycheck :ensure t :init (global-flycheck-mode))
;http://github.com/flycheck/flycheck/blob/master/doc/user/installation.rst#user-content-use-package
 (add-hook 'c-mode-hook 'flycheck-mode) (add-hook 'java-mode-hook 'flycheck-mode)
;http://github.com/flycheck/flycheck/blob/master/doc/user/installation.rst#user-content-use-package
(use-package htmlize :ensure t)
;http://github.com/jcf/emacs.d/blob/master/init-packages.org#user-content-htmlize
(use-package idomenu :ensure t)
(use-package iedit :ensure t) ; http://www.emacswiki.org/emacs/Iedit
(use-package ido :ensure t) (ido-mode) (setq ido-auto-merge-delay-time 9999) (require 'ido)
(use-package imenu-anywhere :ensure t)
(use-package ivy :config (ivy-mode t)) (require 'ivy) (setq ivy-display-style 'fancy)
;^^http://oremacs.com/2015/09/04/ivy-fancy-minibuffer/
(use-package recentf :ensure t) (recentf-mode 1) (setq recentf-max-menu-items 25)
 (defun recentf-ido-find-file () "Find a recent file using Ido." (interactive)
  (let ((file (ido-completing-read recentf-list nil t))) (when file (find-file file))))
; ^^ http://emacsredux.com/blog/2013/04/05/recently-visited-files/
(use-package saveplace :ensure t)
 (setq save-place-file "~/.emacs.d/saveplace") (setq-default save-place t)
(use-package undo-tree :ensure t :init (global-undo-tree-mode))
; https://cestlaz.github.io/posts/using-emacs-16-undo-tree/
(use-package uniquify)
;; http://github.com/jwiegley/use-package/issues/320#issuecomment-186178559
(use-package use-package-chords :ensure t :config (key-chord-mode 1))
(use-package visual-regexp :ensure t)
(use-package which-key :ensure t :config (add-hook 'after-init-hook 'which-key-mode))
(use-package yasnippet :ensure t)
(use-package yasnippet-snippets :ensure t) (yas-global-mode 1)
;http://github.com/AndreaCrotti/yasnippet-snippets#user-content-from-melpa
;; - This is NOT the official Emacs init file for the course IOOPM. - ;;
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; vv More key-bindings starts here.   "(+shift)" refers to a swedish keyboard, but not US dito.
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; vv EMACS shortcut and command vv            vv Shortcut-key vv     In a normal editor* called
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; http://ergoemacs.org/emacs/keystroke_rep.html
(windmove-default-keybindings 'meta)
;; ^^ http://emacsblog.org/2008/05/01/quick-tip-easier-window-switching-in-emacs/
(global-set-key [home] 'crux-move-beginning-of-line);[home] (the key [home] on your keyboard)
(global-set-key "\C-a" 'mark-whole-buffer)        ; ctrl+a           "Select all"
;; ^^ http://ergoemacs.org/emacs/gnu_emacs_keybinding.txt (line 937)
(global-set-key "\M-a" 'back-to-indentation)      ; alt+a            (Almost like [Home])
(global-set-key "\C-b" 'describe-bindings)        ; ctrl+b           ( Settings | Shortcuts... )
(cua-mode t)       ; http://cua.dk/cua.html       ; ctrl+c           "Copy"
(global-set-key "\C-c o" 'ace-window)             ; ctrl+c,o         (Select which window)
(global-set-key "\C-c\C-r" 'recentf-ido-find-file); ctrl+c,ctrl+r    "File | Open Recent"
(global-set-key "\C-\M-d" 'comment-block)         ; ctrl+alt+d       "Comment block"
 (defun comment-block() "Comments the current block." (interactive)
  (let (beg end) (if (region-active-p) (setq beg (region-beginning) end (region-end)))
  (comment-region beg end)))
(global-set-key (kbd "C-M-S-d") 'uncomment-block) ; ctrl+alt+shift+d "Uncomment block"
 (defun uncomment-block() "Uncomments the current block." (interactive)
  (let (beg end) (if (region-active-p) (setq beg (region-beginning) end (region-end)))
  (uncomment-region beg end)))
(global-set-key "\C-e" 'kill-whole-line)          ; ctrl+e            (Delete current line)
(global-set-key "\C-f" 'swiper)     ; ctrl+f           "Find..."
 (defun swiper-region (f &optional initial) (when (and (null initial)
  (use-region-p)) (setq initial (buffer-substring (region-beginning) (region-end)))
  (deactivate-mark t)) (funcall f initial)) (advice-add 'swiper :around #'swiper-region)
; https://piazza.com/class/jkd32onglxi2vb?cid=115
 (define-key ivy-minibuffer-map [escape] 'minibuffer-keyboard-quit)
 (require 'swiper) (define-key swiper-map [escape] 'minibuffer-keyboard-quit)
  (defun minibuffer-keyboard-quit () (interactive)
   (if (and delete-selection-mode transient-mark-mode mark-active) (setq deactivate-mark t)
   (when (get-buffer "*Completions*") (delete-windows-on "*Completions*")) (abort-recursive-edit)))
;http://github.com/abo-abo/swiper/issues/991#issuecomment-355375694
; (global-set-key "\C-f" 'isearch-forward)          ; ctrl+f           "Find..."
;  (define-key isearch-mode-map "\C-f" 'isearch-forward)
;  ; https://stackoverflow.com/questions/7411920/how-to-bind-search-and-search-repeat-to-c-f-in-emacs#7412099
;  (add-hook 'isearch-mode-hook #'find-selected-text) ; This one when text is selected, otherwise swiper below
;   (defun find-selected-text() (when mark-active (let ((region (funcall region-extract-function nil)))
;    (goto-char (region-beginning)) (deactivate-mark) (isearch-update) (isearch-yank-string region))))
;  (define-key minibuffer-local-isearch-map [escape] 'isearch-forward-exit-minibuffer)
;  ; https://stackoverflow.com/questions/17204127/how-to-disable-sticky-highlight-for-repeat-isearch-forward-backward#17222907
;  ; ^^ http://ergoemacs.org/emacs/keystroke_rep.html
;  (global-set-key (kbd "<f3>") 'isearch-repeat-forward)
;  (global-set-key (kbd "S-<f3>") 'isearch-repeat-backward)
; ;http://stackoverflow.com/questions/202803/searching-for-marked-selected-text-in-emacs#32002122
(global-set-key (kbd "M-S-f") 'indent-region)     ; alt+shift+f      "Format Code"
(global-set-key "\M-h" 'er/expand-region)         ; alt+h            (expand region)
(global-set-key "\C-h" 'vr/query-replace)         ; ctrl+h           "Replace..."
(global-set-key "\M-i" 'idomenu)                  ; alt+i            (imenu)
(global-set-key "\C-j" 'avy-goto-word-or-subword-1); ctrl+j          (Jump anywhere
(global-set-key "\C-\M-j" 'avy-goto-char)         ; ctrl+alt+j        on screen in four
(global-set-key "\C-\M-l" 'avy-goto-line)         ; ctrl+alt+l        keystrokes or less.)
(global-set-key "\M-m" 'switch-to-minibuffer)     ; alt+m            (switch to minibuffer)
 (defun switch-to-minibuffer() (interactive) (when (active-minibuffer-window)
  (select-frame-set-input-focus (window-frame (active-minibuffer-window)))
  (select-window (active-minibuffer-window))))
;http://superuser.com/questions/132225/how-to-get-back-to-an-active-minibuffer-prompt-in-emacs-without-the-mouse#132454
(global-set-key "\C-n" 'make-frame)               ; ctrl+n           "New Window"
(global-set-key "\M-n" 'forward-paragraph)        ; alt+n            (next paragraph)
(global-set-key "\C-o" 'find-file)                ; ctrl+o           "Open..."
(global-set-key (kbd "C-S-o") 'find-file-other-frame); ctrl+shift+o  "Open in New Window"
 ; (defun new-window-open () (interactive) (make-frame) (find-file))
(global-set-key "\M-o" 'other-window)             ; alt+o            (switch window)
;; http://www.reddit.com/r/emacs/comments/6dya5h/are_there_any_default_bindings_worth_rebinding/
(global-set-key "\M-p" 'backward-paragraph)       ; alt+p            (previous paragraph)
(global-set-key "\C-s" 'save-buffer)              ; ctrl+s           "Save"
(global-set-key (kbd "C-S-s") 'save-some-buffers) ; ctrl+shift+s!    "Save All"
(global-set-key "\C-\M-s" 'write-file)            ; ctrl+alt+s       "Save As..."
(global-set-key "\C-t" 'toggle-comment)           ; ctrl+t           "Toggle comment"
 (defun toggle-comment() "Toggle comment/uncomment the current line/region." (interactive)
  (let (beg end) (if (region-active-p) (setq beg (region-beginning) end (region-end))
  (setq beg (line-beginning-position) end (line-end-position)))
  (comment-or-uncomment-region beg end)))
; http://stackoverflow.com/questions/9688748/emacs-comment-uncomment-current-line#9697222
(cua-mode t)       ; http://cua.dk/cua.html       ; ctrl+v           "Paste"
(global-set-key "\C-w" 'kill-this-buffer)         ; ctrl+w           "Close"
;; http://superuser.com/questions/354849/emacs-kill-buffer-without-prompt#354878
(cua-mode t)       ; http://cua.dk/cua.html       ; ctrl+x           "Cut"
(global-set-key "\C-x0" 'close-window)            ; ctrl+x, 0        (Emacs: close window)
 (defun close-window () (interactive)
  (condition-case nil (delete-window) (error (save-buffers-kill-terminal))))
(global-set-key "\C-z" 'undo-tree-undo)           ; ctrl+z           "Undo"
(global-set-key (kbd "C-S-z") 'undo-tree-redo)    ; ctrl+shift+z     "Redo"
(global-set-key (kbd "C-.") 'imenu-anywhere)      ; ctrl+.           (Language-aware navigation)
;(defcustom iedit-toggle-key-default "\C-;")      ; ctrl(+shift)+;   "Multiple Cursors" (simple)
; help-command ; (No need to bind - enabled by default) ; f1         "Help"
(global-set-key "\M-/" 'hippie-expand)            ; alt(+shift)+/    (expand if possible)
 (setq hippie-expand-try-functions-list '(try-expand-dabbrev try-expand-dabbrev-all-buffers
 try-complete-file-name-partially try-complete-file-name try-expand-all-abbrevs try-expand-list
 try-expand-dabbrev-from-kill try-expand-line try-complete-lisp-symbol-partially
 try-complete-lisp-symbol))
; (dolist (hook '(prog-mode-hook)) (add-hook hook (lambda ()
;  (interactive) (local-set-key [(meta up)] 'previous-error)))); alt+↑ (previous flycheck-error)
; (dolist (hook '(prog-mode-hook)) (add-hook hook (lambda () (interactive)
;  (local-set-key [(meta down)] 'next-error))))     ; alt+↓            (next flycheck-error)
(global-set-key [(meta f4)] 'close-frame)         ; alt+f4           "Exit/Close"
 (defun close-frame () (interactive)
  (condition-case nil (delete-frame) (error (save-buffers-kill-terminal))))
;http://emacs.stackexchange.com/questions/13485/something-like-delete-frame-but-that-would-also-delete-the-last-frame-like-al#13490
(global-set-key [escape] 'keyboard-clean)
 (defun keyboard-clean () (interactive) (iedit-quit) (isearch-done) (keyboard-escape-quit))
 ; http://stackoverflow.com/questions/12585670/relationship-between-emacs-functions-and-commands#12586078
(define-key minibuffer-local-map [escape] 'abort-recursive-edit)
;http://superuser.com/questions/795763/how-to-make-emacs-quit-the-minibuffer-with-one-press-of-esc#796225
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; * Examples of what I mean by "normal editor" include:
;;   Kate, Noteapad++, GEdit, Notepad2, Crimson Editor, VS Code
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; ^^ More key-bindings ends here. ^^
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; - This is NOT the official Emacs init file for the course IOOPM. - ;;

; Settings for the C programming language:
(add-hook 'c-mode-common-hook (lambda () (progn (setq comment-start "//") (setq comment-end ""))))

(setq kill-buffer-query-functions (delq 'process-kill-buffer-query-function kill-buffer-query-functions))
; http://emacs.stackexchange.com/questions/14509/kill-process-buffer-without-confirmation#14511
(setq custom-file "~/.emacs.d/custom.el") (load custom-file 'noerror)
; ^^ http://emacsblog.org/2008/12/06/quick-tip-detaching-the-custom-file/
(add-hook 'before-save-hook 'do-before-save)
 (defun do-before-save()(delete-trailing-whitespace))
; http://stackoverflow.com/questions/40179200/how-to-use-function-write-region#40179637
(add-hook 'focus-out-hook 'save-buffer)
; ^^ http://emacs.stackexchange.com/questions/265/how-to-auto-save-buffers-when-emacs-loses-focus#266
;; - This is NOT the official Emacs init file for the course IOOPM. - ;;
;;; init.el ends here
